<?php

namespace App\Service;

use App\Exception;

class OrderService {

    private $util;
    private $orderFactory;
    private $productService;
    private $paymentService;

    public function __construct (\App\Util\Util $util,
                                 \App\Service\ProductService $productService,
                                 \App\Service\PaymentService $paymentService,
                                 \App\Factory\OrderFactory $orderFactory) {
        $this->util = $util;
        $this->productService  = $productService;
        $this->paymentService  = $paymentService;
        $this->orderFactory    = $orderFactory;
    }
    
    public function getIdSubProjeto() {
        return $this->orderFactory->getIdSubProjeto();
    }
    
    public function registerIdSubProjeto() {
        return $this->orderFactory->registerIdSubProjeto();
    }
    
    
    public function checkParams($params) {
        
        $subsidiaryCode    = isset($params['subsidiaryCode']) ? $params['subsidiaryCode'] : null;
        $subsidiaryExtCode = isset($params['subsidiaryExtCode']) ? $params['subsidiaryExtCode'] : null;
        
        if(empty($subsidiaryCode) && empty($subsidiaryExtCode)) {
            throw new Exception\ParamsException("Nenhuma Filial foi enviada na consulta. Revise os parametros enviados.");
        }
        
        $generalParams = $this->orderFactory->getFilial($subsidiaryCode, $subsidiaryExtCode);
        
        if((isset($params['subsidiaryExtCode']) && !empty($params['subsidiaryExtCode'])) && empty($generalParams[0])) {
            throw new Exception\ParamsException("Não foi encontrada nenhuma Filial com o codigo externo: {$params['subsidiaryExtCode']}.");
        } 
        
        if(isset($params['subsidiaryCode']) && empty($generalParams[0])) {
            throw new Exception\ParamsException("Não foi encontrada nenhuma Filial com o codigo teknisa: {$params['subsidiaryCode']}.");
        } 
        
        $dtInicial = empty($params['dtInicial']) ? date("d-m-Y", strtotime('-1 day')) : str_replace("/", "-", $params['dtInicial']);
        $dtFinal   = empty($params['dtFinal'])   ? date("d-m-Y", strtotime('-1 day')) : str_replace("/", "-", $params['dtFinal']);
        
        $dtInicial = new \DateTime($dtInicial);
        $dtFinal = new \DateTime($dtFinal);
        $diff  = date_diff($dtInicial, $dtFinal);
        if($diff->d > 5 || $diff->m > 0 ) 
            throw new Exception\ParamsException("Intervalo de datas superior a 5 dias ou data final inválida.");
        
        $dtInicial = date_format($dtInicial, 'd-m-Y H:i:s');
        $dtFinal = date_format($dtFinal, 'd-m-Y H:i:s');
        $tsDtFinal = strtotime($dtFinal);

        if(date("H:i:s", $tsDtFinal) === "00:00:00") {
            $tsDtFinal = strtotime("23:59:59", $tsDtFinal);
            $dtFinal =  date("d-m-Y H:i:s", $tsDtFinal);
        }
        
        if(strtotime($dtInicial) > strtotime($dtFinal))
            throw new Exception\ParamsException("Data/hora inicial superior a data/hora final ou inválida.");
        
        $generalParams[0]['DTINICIAL'] = $dtInicial;
        $generalParams[0]['DTFINAL']   = $dtFinal;
            
        return $generalParams[0];
    }
    
    public function getbranchDetails($cdFilial, $cdFiliExt) {
       if(empty($cdFilial) && empty($cdFiliExt)) {
            throw new Exception\ParamsException("Nenhuma Filial foi enviada na consulta. Revise os parametros enviados.");
        }
        
        $branchDetails = $this->orderFactory->getBranchDetails($cdFilial, $cdFiliExt);
        
        if(empty($branchDetails)) {
            throw new Exception\ParamsException("Erro ao buscar dados da filial. Verifique os parametros enviados na consulta. Código Teknisa: {$cdFilial} | Código Parceiro: {$cdFiliExt}");
        }
        
        if(empty($branchDetails['CDVENDEDOR'])) {
            throw new Exception\ParamsException("Vendedor padrão não parametrizado para Filial.");
        }
        
        if(empty($branchDetails['CDCAIXA'])) {
            throw new Exception\ParamsException("Vendedor padrão sem caixa relacionado. Este cadastro é obrigatório para importação da venda.");
        }
        
        if(empty($branchDetails['CDOPERADOR'])) {
            throw new Exception\ParamsException("Vendedor padrão sem operador relacionado. Este cadastro é obrigatório para importação da venda.");
        }
        
        return $branchDetails;
    }
    
    public function getSales($params) {
        
        $generalParams     = $params;
        $branch            = $this->checkParams($generalParams);
        $subsidiaryCode    = $branch['CDFILIAL'];
        $subsidiaryExtCode = isset($generalParams['subsidiaryExtCode']) ? $generalParams['subsidiaryExtCode'] : null;
        $validateTpEmissao = $this->orderFactory->getTpEmissaoFos($subsidiaryCode);
        $IDAMBTRABNFCE     = !empty($validateTpEmissao) ? $branch['IDAMBTRABNFCE'] : '1';
        $orders            = $this->orderFactory->getSales($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL'], $IDAMBTRABNFCE);
        $salesGroup        = [];
        
        $salesGroup = array(
            'subsidiaryCode'    => $subsidiaryCode,
            'subsidiaryExtCode' => $subsidiaryExtCode,
            'subsidiaryName'    => $branch['NMFILIAL'],
            'sales'             => [],
            'unutilized'        => [],
        );

        foreach($orders as $order) {
            
            $vrTotVenda         = $this->orderFactory->getVrTotalVenda($order['CDFILIAL'], $order['CDCAIXA'], $order['NRSEQVENDA']);
            $vrDescAcrescTotal  = $this->orderFactory->getTotalAcreDesc($order['CDFILIAL'], $order['CDCAIXA'], $order['NRSEQVENDA']);
            $vrTotalImposto     = $this->orderFactory->getTotalImpostos($order['CDFILIAL'], $order['CDCAIXA'], $order['NRSEQVENDA']);
            $vrSaleSubsidized   = $this->orderFactory->getTotalSaleSubsidized($order['CDFILIAL'], $order['CDCAIXA'], $order['NRSEQVENDA'], $order['CDCLIENTE'], $order['DTENTRVENDA']);
            $movSaleCode   = $this->orderFactory->getSaleSubsidized($order['CDFILIAL'], $order['CDCAIXA'], $order['NRSEQVENDA'], $order['CDCLIENTE'], $order['DTENTRVENDA']);
            $hasSubsidizedValue = isset($vrSaleSubsidized['VRMOVCLI']) ? true : false;
            
            $sale  = [
                'noteId'               => $order['NRSEQVENDA'],
                'movSaleCode'          => isset($movSaleCode['NRSEQMOVCLI']) ? $movSaleCode['NRSEQMOVCLI'] : null,
                'noteNumber'           => $order['NRLANCTONFCE'],
                'noteSeries'           => $order['CDSERIENFCE'],
                'chaveSefaz'           => $order['NRACESSONFCE'],
                'protocSefaz'          => $order['NRPROTOCOLONFCE'],
                'cupomNumber'          => $order['NRNOTAFISCALCE'],
                'subsidiaryCode'       => $order['CDFILIAL'],
                'subsidiaryExtCode'    => $subsidiaryExtCode,
                'emissionDate'         => $order['DTEMISSAONFCE'],
                'cashierOpeningDate'   => $order['DTABERTUR'],
                'posCode'              => $order['CDCAIXA'],
                'posDescription'       => $order['NMCAIXA'],
                'cashierCpf'           => isset($order['NRINSCRICAOPARC']) ? $order['NRINSCRICAOPARC'] : null,
                'cashierCode'          => $order['CDOPERADOR'],
                'cashierName'          => $order['NMOPERADOR'],
                'saleDate'             => $order['DTVENDA'],
                'saleStatus'           => $this->getSituVenda($order['IDSITUVENDA']),
                'noteStatus'           => $order['IDSTATUSNFCE'],
                'protoCancSefaz'       => $order['NRPROTOCOLOCANC'],
                'motInutiliza'         => null,
                'saleTotalAmount'      => isset($order['VRTOTVENDA']) ? floatval($this->util->tofloat($order['VRTOTVENDA'])) : 0,
                "saleSubsidizedAmount" => $hasSubsidizedValue ? $this->util->tofloat($vrSaleSubsidized['VRMOVCLI']) : 0,
                'noteTotalAmount'      => $vrTotVenda['VRTOTAL'] ? floatval($this->util->tofloat($vrTotVenda['VRTOTAL'])) : 0,
                'totalDiscountAmount'  => $this->util->tofloat($vrDescAcrescTotal['desconto']),
                'totalAmountAddition'  => $this->util->tofloat($vrDescAcrescTotal['acrescimo']),
                'totalAmountPIS'       => $this->util->tofloat($vrTotalImposto['vrPIS']),
                'totalAmountCOFINS'    => $this->util->tofloat($vrTotalImposto['vrCOFINS']),
                'totalAmountICMS'      => $this->util->tofloat($vrTotalImposto['vrICMS']),
                'consumer'             => [],
                'payments'             => [],
                'itens'                => []
            ];
            
            $itens = $this->productService->getItensVenda($order['CDFILIAL'], $order['CDCAIXA'], $order['NRSEQVENDA']);
            $sale['itens'] = $itens;
            
            $payments = $this->paymentService->getPayments($order['CDFILIAL'], $order['CDCAIXA'], $order['NRSEQVENDA'], $hasSubsidizedValue);
            $sale['payments'] = $payments;
            
            if(!empty($order['NRINSCRCONS']) || !empty($order['CDCONSUMIDOR'])) {
                $consumerData = $this->orderFactory->getconsumerDetails($subsidiaryCode, $order['CDCLIENTE'], $order['CDCONSUMIDOR'], $order['NRINSCRCONS']);
                
                if (!empty($consumerData['NRCELULARCONS']) && strlen($consumerData['NRCELULARCONS']) >= 8) {
                    $phone = $consumerData['NRCELULARCONS'];
                    
                } elseif(!empty($consumerData['NRTELECONS']) && strlen($consumerData['NRTELECONS']) >= 8) {
                    $phone = $consumerData['NRTELECONS'];
                    
                } elseif(!empty($consumerData['NRTELE2CONS']) && strlen($consumerData['NRTELE2CONS']) >= 8) {
                    $phone = $consumerData['NRTELE2CONS'];
                    
                } elseif(!empty($consumerData['NRFAXCONS']) && strlen($consumerData['NRFAXCONS']) >= 8) {
                    $phone = $consumerData['NRFAXCONS'];
                    
                } else {
                    $phone = null;
                }
                
                $consumerDetails = [
                        'consumerName'      => isset($consumerData['NMCONSUMIDOR']) ? $consumerData['NMCONSUMIDOR'] : null,
                        'cpf_cnpj'          => $order['NRINSCRCONS'],
                        'stateRegistration' => null,
                        'email'             => isset($consumerData['DSEMAILCONS']) ? $consumerData['DSEMAILCONS'] : null,
                        'phone'             => $phone,
                        'clientDetails'     => [],
                        'address'           => [
                            'streetName'   => isset($consumerData['DSENDECONS']) ? $consumerData['DSENDECONS'] : null,
                            'streetNumber' => isset($consumerData['NRENDECONS']) ? $consumerData['NRENDECONS'] : null,
                            'neighborhood' => isset($consumerData['NMBAIRCONS']) ? $consumerData['NMBAIRCONS'] : null,
                            'postalCode'   => isset($consumerData['NRCEPCONS']) ? $consumerData['NRCEPCONS'] : null,
                            'city'         => isset($consumerData['NMMUNICIPIO']) ? $consumerData['NMMUNICIPIO'] : null,
                            'state'        => isset($consumerData['SGESTADO']) ? $consumerData['SGESTADO'] : null,
                            'country'      => isset($consumerData['NMPAIS']) ? $consumerData['NMPAIS'] : null,
                            ]
                    ];
                    
                if($consumerData['IDTPIJURCLIE'] !== 'O') {
                    $clientDetails = [
                            'cpf_cnpj'          => $consumerData['NRINSJURCLIE'],
                            'corporateName'     => $consumerData['NMRAZSOCCLIE'],
                            'TradeName'         => $consumerData['NMFANTCLIE'],
                            'stateRegistration' => $consumerData['NRINSESTCLIE'],
                            'CityRegistration'  => $consumerData['NRINSMUNICLIE']
                        ];
                    $consumerDetails['clientDetails'] = $clientDetails;
                }
    
                $sale['consumer'] = $consumerDetails;
            }
            
            array_push($salesGroup['sales'], $sale);
        }
        
        $notasInutilizadas = $this->orderFactory->getUnutilizedNotes($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);
        if(!empty($notasInutilizadas)) {
            $arrayunutilized = [];
            foreach($notasInutilizadas as $nota) {
                $unutilized = [
                        'subsidiaryCode'           =>  $nota['CDFILIAL'],
                        'cupomNumber'              =>  $nota['NRNOTAFISCALCE'],
                        'noteSeries'               =>  $nota['CDSERIENFCE'],
                        'noteStatus'               =>  $nota['IDSTATUSNFCE'],
                        'noteDate'                 =>  $nota['DTENTRVENDA'],
                        'inutilizationDate'        =>  $nota['DTHRINUTNFCE'],
                        'inutilizationDescription' =>  $nota['DSOBSINUTNFCE']
                ];
                    
                array_push($arrayunutilized,$unutilized);
            }
            array_push($salesGroup['unutilized'], json_decode(json_encode($arrayunutilized), true));
        }
        return $salesGroup;
    }
    
    public function getSituVenda($idSituVenda) {
        $statusVenda = array(
            'O' => 'Concluida',
            'C' => 'Cancelada'
            );
            
        return isset($statusVenda[$idSituVenda]) ? $statusVenda[$idSituVenda] : null;
    }
    
    public function getReduction($params) {
        $generalParams = $params;
        
        $branch = $this->checkParams($generalParams);
        $subsidiaryCode = $branch['CDFILIAL'];
        
        $closingType = $this->orderFactory->getClosingType($subsidiaryCode);

        if($closingType['IDTPAPUFIL'] === 'S') {
            $cdTpSangria = $this->orderFactory->getTipoSangria($subsidiaryCode);
            
            if(empty($cdTpSangria))
                throw new Exception\ParamsException("Tipo de sangria de fechamento não parametrizado para a filial {$subsidiaryCode}.");
                
            $sangrias = $this->orderFactory->closingPosBySangria($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL'], $cdTpSangria);
        }else{
            $sangrias = $this->orderFactory->closingPosByVenda($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);
            
        }
        if(empty($sangrias))
                throw new Exception\ParamsException("Não a dados a serem listados.");
            
        $arraySangrias = [
                    'subsidiaryCode' => $subsidiaryCode,
                    'subsidiaryName' => $branch['NMFILIAL'],
                    'closing'    => []
        ];
            
        foreach($sangrias as $sangria) {

            $paymentDetails = $this->orderFactory->getPaymentInfo($sangria['CDTIPORECE']);
            
            if($paymentDetails['IDTIPORECE'] == "1" || $paymentDetails['IDTIPORECE'] == "2") {
                $cardType = $paymentDetails['IDTIPORECE'];
            }elseif($paymentDetails['IDTIPORECE'] == 'G' || $paymentDetails['IDTIPORECE'] == 'F') {
                $cardType = "3";
            }else{
                $cardType = null;
            }
                
            if(empty($cardType))
                $cardDescription = $this->paymentService->getCardType($cardType);
            
            $receipt = array(
                    'movimentDate'    => str_replace("-", "/",$sangria['DTABERCAIX']),
                    'payment'         => $paymentDetails['NMTIPORECE'],
                    'cardType'        => !empty($cardType) ? $cardType : '',
                    'cardDescription' => !empty($cardType) ? $cardDescription : '',
                    'paymentCode'     => $paymentDetails['CDTIPORECE'],
                    'externalCode'    => $paymentDetails['CDTPRECEINTE'],
                    'amount'          => floatval($this->util->tofloat($sangria['VRMOVIVEND']))
                    );
            array_push($arraySangrias['closing'], $receipt);
        }   
        return $arraySangrias;
        
    }
    
    public function getStockExit($params) {
        $generalParams = $params;
        
        $branch = $this->checkParams($generalParams);
        $subsidiaryCode = $branch['CDFILIAL'];
        
        $stock = [
                'subsidiaryCode' => $subsidiaryCode,
                'subsidiaryName' => $branch['NMFILIAL'],
                'stockRequest'   => []
            
            ];
            
        $nrLancEstq = [];
        
        $planRequests = $this->orderFactory->getPlanRequest($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);

        if(!empty($planRequests)) {
            array_push($nrLancEstq, $planRequests);
        }
        
        $saleRequests = $this->orderFactory->getSaleRequest($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);
        if(!empty($saleRequests)) {
            array_push($nrLancEstq, $saleRequests);
        }
            
        $movEst = [];
        
        foreach($nrLancEstq as $modLanc) {
            foreach($modLanc as $lanc) {
            
                $arrayItens = $this->orderFactory->getMovEstoq($subsidiaryCode, $lanc['NRLANCESTQ']);
                $reqEst = array (
                            'requisitionId' => $lanc['NRLANCESTQ'],
                            'date'          => $this->orderFactory->getDateMovEst($subsidiaryCode, $lanc['NRLANCESTQ']),
                            'itens'         => []
                        );
                foreach($arrayItens as $itens) {
                    $qtd = $this->util->tofloatQtd($this->recalculateQtd($itens));
                    $qtd = round($qtd, 4);
                    $itemDetails = array(
                                    'requisitionId' => $itens['NRLANCESTQ'],
                                    'cupomNumber'   => !empty($itens['NRNOTAFISCALCE']) ? $itens['NRNOTAFISCALCE'] : '',
                                    'name'          => $itens['NMPRODUTO'],
                                    'code'          => $itens['CDARVPROD'],
                                    'externalCode'  => $itens['CDPRODINTE'],
                                    'unit'          => $itens['SGUNIDADE'],
                                    'quantity'      => $qtd
                                 );
                    array_push($reqEst['itens'], $itemDetails);
                }
                array_push($movEst, $reqEst);
            }
        }
        
        if(!empty($movEst))
            $stock['stockRequest'] = $movEst;
        
        return $stock;
        
    }
    
    private function recalculateQtd($itens) {
        if ((empty($itens['QTDCOMP'])) || (empty($itens['QTPRODVEND']))) {
            return $itens['QTLANCTOEST'];
        }else{
            $qtdComp = $this->util->tofloat($itens['QTDCOMP']);
            $qtdVend = $this->util->tofloat($itens['QTPRODVEND']);
            return $qtdComp * $qtdVend;
        }
        
    }
    
    public function getDayCloses($params) {
        $generalParams = $params;
        
        if((isset($generalParams['date'])) && (!empty($generalParams['date']))) {
            $generalParams['date'] = str_replace("/", "-", $generalParams['date']);
        } else {
            $generalParams['date'] = date("d-m-Y", strtotime('-1 day'));
        }
        
        $branch         = $this->checkParams($generalParams);
        $subsidiaryCode = $branch['CDFILIAL'];
        $movimentDate   = $generalParams['date'];
        
        $dayCloses  = $this->orderFactory->getDayCloses($subsidiaryCode, $movimentDate);
        
        $return = [
            'subsidiaryCode'     => $subsidiaryCode,
            'subsidiaryExtCode'  => isset($generalParams['subsidiaryExtCode']) ? $generalParams['subsidiaryExtCode'] : null,
            'subsidiaryName'     => $branch['NMFILIAL'],
            'cashierOpeningDate' => $generalParams['date'],
            'dayCloses'          => []
        ];
        
        if(!empty($dayCloses)) {
            foreach($dayCloses as $close){
                $arrClose = [
                    'cashierOpeningDate' => $close['DTABERCAIX'],
                    'cashierClosingDate' => $close['DTFECHCAIX'],
                    'posCode'            => $close['CDCAIXA'],
                    'cashierCpf'         => isset($close['NRINSCRICAOPARC']) ? $close['NRINSCRICAOPARC'] : null,
                    'cashierCode'        => $close['CDOPERADOR'],
                    'cashierName'        => $close['NMOPERADOR'],
                ];
                array_push($return['dayCloses'], $arrClose);
            }
        }
        
        return $return;
    }
    
    public function getClosingSummary($params) {
        $generalParams = $params;
        
        if((isset($generalParams['cashierClosingDate'])) && (!empty($generalParams['cashierClosingDate']))) {
            $generalParams['dtInicial'] = str_replace("/", "-", $generalParams['cashierClosingDate']);
            $generalParams['dtFinal']   = str_replace("/", "-", $generalParams['cashierClosingDate']);
        } else {
            $generalParams['dtInicial'] = date("d-m-Y");
            $generalParams['dtFinal']   = date("d-m-Y");
        }
        
        $branch         = $this->checkParams($generalParams);
        $subsidiaryCode = $branch['CDFILIAL'];
        $tsDtInicial    = strtotime($branch['DTINICIAL']);

        if(date("H:i:s", $tsDtInicial) !== "00:00:00") {
            $tsDtInicial = strtotime("00:00:00", $tsDtInicial);
            $branch['DTINICIAL'] =  date("d-m-Y H:i:s", $tsDtInicial);
        }

        $aberturDetails = $this->orderFactory->getAbertur($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);
        
        if(count($aberturDetails) < 1 )
            throw new Exception\ParamsException("Não foi encontrado movimentação de caixa para a data e hora informado.");
        
        $return = [
            'subsidiaryCode'     => $subsidiaryCode,
            'subsidiaryExtCode'  => isset($generalParams['subsidiaryExtCode']) ? $generalParams['subsidiaryExtCode'] : null,
            'subsidiaryName'     => $branch['NMFILIAL'],
            'dayClosing'          => []
            ];
        
    
        $closingType = $this->orderFactory->getClosingType($subsidiaryCode);
        
        foreach($aberturDetails as $moviment) {
            
            $subTotal    = '0';
            $total       = '0';
            $sangrias    = '0';
            $supply = $this->orderFactory->getCashSupply($moviment['CDFILIAL'], $moviment['CDCAIXA'], $moviment['DTABERCAIX']);

            $arrClose = [
                'cashierOpeningDate' => $moviment['DTABERCAIX'],
                'cashierClosingDate' => $moviment['DTFECHCAIX'],
                'posCode'            => $moviment['CDCAIXA'],
                'cashierCpf'         => isset($moviment['NRINSCRICAOPARC']) ? $moviment['NRINSCRICAOPARC'] : null,
                'cashierCode'        => $moviment['CDOPERADOR'],
                'cashierName'        => $moviment['NMOPERADOR'],
                'cashSupply'         => isset($supply['VRMOVIVEND']) ? $this->util->tofloat($supply['VRMOVIVEND']) : 0,
                'cashierSummary'     => [
                            'closing'        => [],
                            'cashWithdrawal' => []
                        ]   
            ];

            if($closingType['IDTPAPUFIL'] === 'S') {        
                $closingSummary  = $this->orderFactory->getClosingSummary($moviment['CDFILIAL'], $moviment['CDCAIXA'], $moviment['DTABERCAIX']);
            } else {
                 $closingSummary  = $this->orderFactory->getClosingSummaryBySale($moviment['CDFILIAL'], $moviment['CDCAIXA'], $moviment['DTABERCAIX']);
            }
            if(!empty($closingSummary)) {
                foreach($closingSummary as $closing) {
                    $receipt = $this->orderFactory->getPaymentInfo($closing['CDTIPORECE']);
                    if(empty($receipt))
                        throw new Exception\ParamsException("Tipo de recebimento não encontrado");
                    
                    $cardType = $this->getCardType($receipt['IDTIPORECE']);
                    $summary = [
                            'payment'         => $receipt['NMTIPORECE'],
                            'cardType'        => isset($cardType) ? $cardType : '',
                            'cardDescription' => isset($cardType) ? $this->paymentService->getCardType($cardType) : '',
                            'paymentCode'     => $receipt['CDTIPORECE'],
                            'externalCode'    => $receipt['CDTPRECEINTE'],
                            'amount'          => $this->util->tofloat($closing['VRMOVIVEND']),
                        ];
                    $subTotal = $this->util->add($subTotal, $closing['VRMOVIVEND']);
                    array_push($arrClose['cashierSummary']['closing'], $summary);
                }
            }
            
            $withdrawals = $this->orderFactory->getCashWithdrawal($moviment['CDFILIAL'], $moviment['CDCAIXA'], $moviment['DTABERCAIX']);
            if(!empty($withdrawals)) {
                foreach($withdrawals as $cashWithdrawal) {
                    $receiptWithdrawal = $this->orderFactory->getPaymentInfo($cashWithdrawal['CDTIPORECE']);
                    if(empty($receiptWithdrawal))
                        throw new Exception\ParamsException("Tipo de recebimento não encontrado");
                    
                    $cardType = $this->getCardType($receiptWithdrawal['IDTIPORECE']);
                    $Withdrawal = [
                            'withdrawalTypeCode' => $cashWithdrawal['CDTPSANGRIA'],
                            'withdrawalType'     => $cashWithdrawal['NMTPSANGRIA'],
                            'payment'            => $receiptWithdrawal['NMTIPORECE'],
                            'cardType'           => isset($cardType) ? $cardType : '',
                            'cardDescription'    => isset($cardType) ? $this->paymentService->getCardType($cardType) : '',
                            'paymentCode'        => $receiptWithdrawal['CDTIPORECE'],
                            'externalCode'       => $receiptWithdrawal['CDTPRECEINTE'],
                            'amount'             => $this->util->tofloat($cashWithdrawal['VRMOVIVEND']),
                            'notes'              => $cashWithdrawal['DSOBSSANGRIACX']
                        ];
                        $sangrias = $this->util->add($sangrias, $cashWithdrawal['VRMOVIVEND']);
                        array_push($arrClose['cashierSummary']['cashWithdrawal'], $Withdrawal);
                }
            }
            if($closingType['IDTPAPUFIL'] === 'S'){
                $totalWithWithdrawal    = $subTotal;
                $totalWithoutWithdrawal = $this->util->add($subTotal, $sangrias);
            } else {
                $totalWithWithdrawal    = $subTotal > $sangrias ? $this->util->subtract($subTotal, $sangrias) : 0;
                $totalWithoutWithdrawal = $subTotal;
            }
            $arrClose['totalWithWithdrawal']    = $this->util->tofloat($totalWithWithdrawal);
            $arrClose['totalWithoutWithdrawal'] = $this->util->tofloat($totalWithoutWithdrawal);
            $arrClose['total']                  = $sangrias < $subTotal ? $this->util->subtract($arrClose['totalWithoutWithdrawal'], $sangrias) : 0;
            array_push($return['dayClosing'], $arrClose);
        }
        return $return;
    }
    
    private function getCardType($paymentType) {
        if($paymentType == "1" || $paymentType == "2") {
            $cardType = $paymentType;
        } elseif ($paymentType == 'G' || $paymentType == 'F') {
            $cardType = "3";
        } else {
            $cardType = null;
        }
        return $cardType;
    }
    
    public function validateDuplicatedSale($cdFilial, $cupomNumber, $chaveSefaz) {
        return $this->orderFactory->validateDuplicatedSale($cdFilial, $cupomNumber, $chaveSefaz);
        
    }
    
    public function processSalesImport($sale, $subsidiaryCode, $generalParams, $insertSubsidizedConsumption, $idSubProjeto) {
        
        $dtAberCaix = $sale['cashierOpeningDate'];
        $cdFilial   = $subsidiaryCode;
        $cdCaixa    = isset($sale['posCode']) ? $sale['posCode'] : $generalParams['CDCAIXA'];
        $cdLoja     = $generalParams['CDLOJA'];
        $cdOperador = $generalParams['CDOPERADOR'];
        $nrOrg      = $generalParams['NRORG'];
        
        $posStatus = $this->orderFactory->getPosStatus($cdFilial, $cdCaixa, $dtAberCaix);
        
        if(empty($posStatus['status'])) {
            $dtMovTurCaix = $dtAberCaix;
            $dtAberCaix   = $dtAberCaix.' 00:00:01';
            $this->orderFactory->openingPos($cdFilial, $cdCaixa, $dtAberCaix, $dtMovTurCaix, $cdOperador, $nrOrg);
        }
        
        if($posStatus['status'] == 'Fechado' && $posStatus['allowsReopening']) {
            $dtAberCaix = $posStatus['openingPos'];
            $this->orderFactory->reopenPos($cdFilial, $cdCaixa, $posStatus['openingPos'], $cdOperador);
        }
                   
        if($posStatus['status'] == 'Fechado' && !$posStatus['allowsReopening']) {
            $saleMonth = date('m', strtotime($dtAberCaix));
            if($saleMonth != date('m')) {
                $day = date('d-m-Y');
                throw new Exception\ParamsException(
                    "Erro:Não foi possivel importar a venda: Cupom {$sale['cupomNumber']} - Filial {$subsidiaryCode} - Data da venda {$sale['saleDate']}.<br>
                     Motivo: Data de movimento com fechamento de cupom aprovado e data corrente em periodo fiscal diferente. DtMovimento: {$dtAberCaix} | Dt Corrente: {$day}.<br>
                     Correção: Necessário desaprovar o fechamento de cupom para o dia: {$dtAberCaix} - Filial: {$subsidiaryCode} - Caixa: {$cdCaixa}.");
            }
            
            $message = "O caixa do dia {$dtAberCaix} foi fechado como ultimo fechamento do dia. Importando Cupom {$cupomNumber} - Serie {$noteSeries} na data corrente ". date('d-m-Y');
            $this->util->writeLog('execution', '', date("d-m-Y H:i:s") . " - " . $message);
            $dtMovTurCaix = date('d/m/Y');
            $dtAberCaix   = date('d/m/Y H:i:s');
            $this->orderFactory->openingPos($cdFilial, $cdCaixa, $dtAberCaix, $dtMovTurCaix, $cdOperador, $nrOrg);
        }
        
        if($posStatus['status'] == 'Aberto') {
            $dtAberCaix = $posStatus['openingPos'];
        }
        
        $xmlDecode = base64_decode($sale['xml']);
        $xml = json_decode(json_encode(simplexml_load_string($xmlDecode)), true);

        $isNFCe = $generalParams['IDTPEMISSAOFOS'] == 'FNC'? true : false;
        
        if ($isNFCe && isset($xml['infNFe'])) {
            $noteDetails = $xml['infNFe'];
            $noteDetails['EMISSAO'] = isset($xml['protNFe']) ? $xml['protNFe'] : null; 
        } elseif ($isNFCe && isset($xml['infCFe'])) {
            $noteDetails = $xml['infCFe'];
            $noteDetails['EMISSAO'] = null; 
        } else {
            throw new Exception\ParamsException("Tipo de emissão de venda desconhecido ou inválido. Tipo emissão do caixa: {$generalParams['IDTPEMISSAOFOS']}");
        }
        
        $idStatusNFCe    = 'P';
        $idXmlValidado   = 'N';
        $nrProtcoloNFCe  = null;
        $dsObsStatusNFCe = null;
        $dateProtoc      = null;
        $dtHrProtoNFCe   = null;
        
        $nrSeqVenda = $this->util->newCode('PEDIDOVENDA'.$cdFilial.$cdCaixa, 10);
        $dtEntrVenda = date('d-m-Y', strtotime($sale['saleDate']));
        $isApprovedNFCe = !empty($noteDetails['EMISSAO']) ? true : false;
        
        if($isNFCe) {
            if($isApprovedNFCe) {
                $idStatusNFCe    = 'A';
                $nrProtcoloNFCe  = $noteDetails['EMISSAO']['infProt']['nProt'];
                $dsObsStatusNFCe = $noteDetails['EMISSAO']['infProt']['xMotivo'];
                $dateProtoc      = strtotime($noteDetails['EMISSAO']['infProt']['dhRecbto']);
                $dtHrProtoNFCe   = date('d-m-Y H:i:s', $dateProtoc);
                $idXmlValidado   = 'S';
            }
        }
        
        $this->orderFactory->insertSale($sale, $nrSeqVenda, $cdFilial, $cdLoja, $cdCaixa, $cdOperador, $nrOrg, $generalParams, $dtAberCaix,
                                        $dtEntrVenda, $xmlDecode, $noteDetails, $isNFCe, $isApprovedNFCe, $idStatusNFCe, $nrProtcoloNFCe, 
                                        $dsObsStatusNFCe, $dateProtoc, $dtHrProtoNFCe, $idXmlValidado, $idSubProjeto
                                        );
                                        
        $this->orderFactory->insertXML($cdFilial, $cdCaixa, $nrSeqVenda, $xmlDecode, $nrOrg);
        
        if(empty($sale['itens'])) {
            throw new Exception\ParamsException("É obrigatório o envio dos produto que compoem a venda.");
        }
        
        $qtdItens    = count($sale['itens']);
        $qtdItensXml = count($noteDetails['det']);
        
        if ($qtdItens != $qtdItensXml) {
            throw new Exception\ParamsException(
                "Divergencia na quantidade de produtos no payload em relação ao XML. Qtd Payload: {$qtdItens} | Qtd xml: {$qtdItensXml}");
        }
        
        
        // Processa os Produtos
        $nrSequitVend = 0;
        foreach($sale['itens'] as $product) {
            $nrSequitVend++;
            $this->productService->processProduct($cdFilial, $cdLoja, $cdCaixa, $cdOperador, $generalParams, $dtAberCaix, $product, $nrSeqVenda, $nrOrg, $nrSequitVend);
        }
        
        // Processa os Recebimentos
        if(empty($sale['payments'])) {
             throw new Exception\ParamsException("Não foi enviado nenhum pagamento para a venda, Estes dados são obrigatórios.");
        }
        
        $paymentCodeSubsidized = null;
        $countNrseq = $this->orderFactory->getSeqMovCaixa($cdFilial, $cdCaixa, $dtAberCaix);
        $countNrseq = isset($countNrseq['NRSEQUMOVI']) ? $countNrseq['NRSEQUMOVI'] : 0;
        
        foreach($sale['payments'] as $payment) {
            
            $paymentDetails = $this->paymentService->processPayment($payment);
            $nrSequMovi = $this->util->newCode('MOVCAIXA'.$cdFilial.$cdCaixa.$dtAberCaix, 10);
            $countNrseq++;
            $nrSequMovi = $nrSequMovi < $countNrseq ? $countNrseq : $nrSequMovi;
            
            if($paymentDetails['IDTIPORECE'] == 'A') {
                $paymentCodeSubsidized = $paymentDetails['CDTIPORECE'];
            }
            
            if($payment['movimentType'] == 'S' || $payment['Value'] < 0.01) {
                throw new Exception\ParamsException("Valor não atende ao valor mínimo de pagamento. mínimo esperado: 0.01 | Valor recebido: {$payment['Value']}.");
            }
            
            $this->orderFactory->insertMovCaixa($payment, $paymentDetails, $nrSeqVenda, $cdFilial, $cdLoja, $cdCaixa, $cdOperador, $nrOrg, $generalParams, $dtAberCaix, $dtEntrVenda, $nrSequMovi);
        }
        
        if(isset($movSaleCode) && !empty($movSaleCode)) {
            $movClie = $this->orderFactory->getConsumptionMovClie($cdFilial, $cdInteMovSale);
            
            if(!empty($movClie)) {
                throw new Exception\ParamsException("O consumo identicado no codigo: {$cdInteMovSale} ainda não foi sincronizado com o Oracle ou é inválido. Favor reenviar esta requisição mais tarde.");
            }
            
            $cdCliente    = $movClie['CDCLIENTE'];
            $cdCusClie    = $movClie['CDCCUSCLIE'];
            $cdConsumidor = $movClie['CDCONSUMIDOR'];
                
            $this->orderFactory->updateSale($cdFilial, $cdCaixa, $nrSeqVenda, $dtAberCaix, $cdCliente, $cdConsumidor, $cdCusClie);
            $this->orderFactory->updateMovClie($cdFilial, $cdCaixa, $nrSeqVenda, $dtAberCaix, $nrSequMovi, $cdTipoRece, $cdCliente, $cdConsumidor, $cdInteMovSale);
            
        }
        
        if($posStatus['status'] == 'Fechado' && $posStatus['allowsReopening']) {
            $this->orderFactory->closePos($cdFilial, $cdCaixa, $posStatus['openingPos'], $cdOperador);
        }
        
        $this->validateSale($cdFilial, $cdCaixa, $generalParams, $nrSeqVenda, $noteDetails, $isNFCe);
    }
    
    private function validateSale($cdFilial, $cdCaixa, $generalParams, $nrSeqVenda, $noteDetails, $isNFCe) {
        $emitente      = $noteDetails['emit'];
        $totalizadores = $noteDetails['total'];
        $vrIcms        = $totalizadores['ICMSTot']['vICMS'];
        $vrPis         = $totalizadores['ICMSTot']['vPIS'];
        $vrCofins      = $totalizadores['ICMSTot']['vCOFINS'];
        $totalProdutos = isset($totalizadores['ICMSTot']['vNF']) ? $totalizadores['ICMSTot']['vNF'] : $totalizadores['vCFe'];
        $totalPago     = 0;
       
        if($emitente['CNPJ'] != $generalParams['NRINSJURIEST']){
            throw new Exception\ParamsException("CNPJ do xml diferente do CNPJ vinculado a IE.");
        }
            
        if($emitente['IE'] != $generalParams['CDINSCESTA']) {
            throw new Exception\ParamsException("IE do xml diferente da IE vinculado a Unidade.");
        }
        
        if($isNFCe) {
            if($emitente['CRT'] != $generalParams['CDSITUCRT']) {
                throw new Exception\ParamsException("CRT do xml diferente do CRT parametrizado para empresa.");
            }
            $pagamentos = $noteDetails['pag'];
            
            foreach($pagamentos['detPag'] as $pag) {
                $totalPago = isset($pag['vPag']) ? bcadd($pag['vPag'], $totalPago, 2) : $totalPago;

            }
            
            $totalPago = bcsub($totalPago, $pagamentos['vTroco'], 2);
            
        } else {
            if($emitente['cRegTrib'] != $generalParams['CDSITUCRT']) {
                throw new Exception\ParamsException("CRT do xml diferente do CRT parametrizado para empresa.");
            }
            $pagamentos = $noteDetails['pgto'];
            
            foreach($pagamentos as $pag) {
                $totalPago = isset($pag['vPag']) ? bcadd($pag['vPag'], $totalPago, 2) : $totalPago;

            }
            
            $totalPago = bcsub($totalPago, $pagamentos['vTroco'], 2);
        }
        
        $vrTotalVenda  = $this->orderFactory->getVrTotalVenda($cdFilial, $cdCaixa, $nrSeqVenda);
        $impostos      = $this->orderFactory->getTotalImpostos($cdFilial, $cdCaixa, $nrSeqVenda);  
        $vrTotalIcms   = $this->util->tofloat($impostos['vrICMS']);
        $vrTotalPis    = $this->util->tofloat($impostos['vrPIS']);
        $vrTotalCofins = $this->util->tofloat($impostos['vrCOFINS']);
        
        if(bccomp($vrTotalIcms, $vrIcms, 2)) {
            throw new Exception\ParamsException("Divergência do valor total do ICMS no XML para o valor informado. xml: {$vrIcms} | Informado: {$vrTotalIcms}");
        }
        
        if(bccomp($vrTotalPis, $vrPis, 2)) {
            throw new Exception\ParamsException("Divergência do valor total do PIS no XML para o valor informado. xml: {$vrPis} | Informado: {$vrTotalPis}");
        }
        
        if(bccomp($vrTotalCofins, $vrCofins, 2)) {
            throw new Exception\ParamsException("Divergência do valor total do COFINS no XML para o valor informado. xml: {$vrCofins} | Informado: {$vrTotalCofins}");
        }
        
        if(bccomp($totalProdutos, $vrTotalVenda, 2)) {
            throw new Exception\ParamsException("Divergência do valor total dos produtos no XML em relação aos valores informados. xml: {$totalProdutos} | Informado: {$vrTotalVenda}");
        }
        
    }
    
    public function getProductTaxes($params) {
        
        $subsidiaryCode    = isset($params['subsidiaryCode']) ? $params['subsidiaryCode'] : null;
        $subsidiaryExtCode = isset($params['subsidiaryExtCode']) ? $params['subsidiaryExtCode'] : null;
        $products          = isset($params['productsCode']) ? $params['productsCode'] : null;
        
        if(empty($subsidiaryCode) && empty($subsidiaryExtCode)) {
            throw new Exception\ParamsException("Nenhuma Filial foi enviada na consulta. Revise os parametros enviados.");
        }
        
        $generalParams = $this->orderFactory->getFilial($subsidiaryCode, $subsidiaryExtCode);

        if((isset($params['subsidiaryExtCode']) && !empty($params['subsidiaryExtCode'])) && empty($generalParams[0])) {
            throw new Exception\ParamsException("Não foi encontrada nenhuma Filial com o codigo externo: {$params['subsidiaryExtCode']}.");
        } 
        
        if(isset($params['subsidiaryCode']) && empty($generalParams[0])) {
            throw new Exception\ParamsException("Não foi encontrada nenhuma Filial com o codigo teknisa: {$params['subsidiaryCode']}.");
        }
        
        if(empty($products)) {
            throw new Exception\ParamsException("Nenhum produto enviado para consulta. ");
        }
        
        $arrProducts = [];
        foreach($products as $product) {
            $cdProduct = $this->productService->getCodeProduct($product);
            if(empty($cdProduct)) {
                 throw new Exception\ParamsException("Produto {$product} não encontrado. ");
            }
            array_push($arrProducts, $cdProduct['CDPRODUTO']);
        }

        $taxes = $this->productService->getProductTaxes($generalParams[0]['CDFILIAL'], $arrProducts);

        $return = [
            'subsidiaryCode'    => $generalParams[0]['CDFILIAL'],
            'subsidiaryExtCode' => $subsidiaryExtCode,
            'taxes'             => []
        ];
        
        foreach($taxes as $taxProd) {
            $arr = [
                'productName'                => $taxProd['NMPRODUTO'],
                'productCode'                => $taxProd['CDARVPROD'],
                'externalCode'               => $taxProd['CDPRODINTE'],
                'taxCode'                    => $taxProd['CDIMPOSTO'],
                'aliquot'                    => isset($taxProd['VRPEALIMPFIS']) ? $this->util->tofloat($taxProd['VRPEALIMPFIS']) : 0,
                'icmsReductionPercentage'    => isset($taxProd['VRPERCREDUCAO']) ? $this->util->tofloat($taxProd['VRPERCREDUCAO']) : 0,
                'icmsCalculationBasis'       => $taxProd['IDMODALBASECALC'],
                'incidencePisCofins'         => $taxProd['IDINCIDEPISCOF'], 
                'aliqIBPTF'                  => isset($taxProd['VRALIQIBPT']) ? $this->util->tofloat($taxProd['VRALIQIBPT']) : 0,
                'aliqIBPTE'                  => isset($taxProd['VRALIQIBPTES']) ? $this->util->tofloat($taxProd['VRALIQIBPTES']) : 0,
                'cfopCode'                   => $taxProd['CDCFOPPFIS'],
                'cstIcms'                    => $taxProd['CDCSTICMS'],
                'cstPisCofins'               => $taxProd['CDCSTPISCOF'],
                'pisTaxRate'                 => isset($taxProd['VRALIQPIS']) ? $this->util->tofloat($taxProd['VRALIQPIS']) : 0,
                'cofinsTaxRate'              => isset($taxProd['VRALIQCOFINS']) ? $this->util->tofloat($taxProd['VRALIQCOFINS']) : 0,
                'fcpTaxRate'                 => isset($taxProd['VRALIQFCP']) ? $this->util->tofloat($taxProd['VRALIQFCP']) : 0,
                'effectiveIcmsTaxRate'       => isset($taxProd['VRPEALIMPFISEF']) ? $this->util->tofloat($taxProd['VRPEALIMPFISEF']) : 0,
                'effectiveIcmsRedPercentage' => isset($taxProd['VRPERCREDUCAOEF']) ? $this->util->tofloat($taxProd['VRPERCREDUCAOEF']) : 0,
                'icmsExemptionRate'          => isset($taxProd['VRALIQICMSDES']) ? $this->util->tofloat($taxProd['VRALIQICMSDES']) : 0,
                'fiscalBenefitCode'          => $taxProd['CDCBENEF'] 
             ];
            
            array_push($return['taxes'], $arr);
        }
        return $return;
    }
    
    public function setProductTaxes($params) {
        
        $subsidiaryCode    = isset($params['subsidiaryCode']) ? $params['subsidiaryCode'] : null;
        $subsidiaryExtCode = isset($params['subsidiaryExtCode']) ? $params['subsidiaryExtCode'] : null;
        $products          = isset($params['taxes']) ? $params['taxes'] : null;
        
        if(empty($subsidiaryCode) && empty($subsidiaryExtCode)) {
            throw new Exception\ParamsException("Nenhuma Filial foi enviada na consulta. Revise os parametros enviados.");
        }
        
        $branchDetails = $this->orderFactory->getBranchDetails($subsidiaryCode, $subsidiaryExtCode);

        if((isset($params['subsidiaryExtCode']) && !empty($params['subsidiaryExtCode'])) && empty($branchDetails)) {
            throw new Exception\ParamsException("Não foi encontrada nenhuma Filial com o codigo externo: {$params['subsidiaryExtCode']}.");
        } 
        
        if(isset($params['subsidiaryCode']) && empty($branchDetails)) {
            throw new Exception\ParamsException("Não foi encontrada nenhuma Filial com o codigo teknisa: {$params['subsidiaryCode']}.");
        }
        
        if(empty($products)) {
            throw new Exception\ParamsException("Nenhum produto enviado para cadastro de imposto.");
        }
        
        foreach($products as $product) {
            $cdProduto = $this->productService->getCodeProduct($product['productCode']);
            if(empty($cdProduto)) {
                 throw new Exception\ParamsException("Produto {$product} não encontrado.");
            }
            
            $this->validateTaxesByCRT($branchDetails['CDSITUCRT'], $product);

            if (((($product['cstIcms'] != '500') && ($product['cstIcms'] != '60')) || $product['cfopCode'] != '5405') && $product['taxCode'] == '99') {
                throw new Exception\ParamsException("Produto {$product['productName']} codigo: {$product['productCode']} enviado com imposto 99 - ST e informações CST/CSOSN, CST ou CFOP de ICMS.");
            }
            
            if (((($product['cstIcms'] == '500') || ($product['cstIcms'] == '60')) || $product['cfopCode'] == '5405') && $product['taxCode'] != '99') {
                throw new Exception\ParamsException("Produto {$product['productName']} codigo: {$product['productCode']} enviado com imposto {$product['taxCode']} e informações CST/CSOSN, CST ou CFOP de ST.");
            }
            
            $this->productService->registerProductTaxes($branchDetails, $product, $cdProduto);
        }
    }
    
    private function validateTaxesByCRT($CRT, $productTaxes) {
        
        $cfop        = ['5101', '5102', '5103', '5104', '5115', '5405', '5653', '5656', '5667', '5933'];
        $cstPisCof   = ['01', '02', '03', '04', '05', '06', '07', '08', '09', '49', '99'];
        $cstSN       = ['40', '101', '102', '103', '300', '400', '500', '900'];
        $cstRN       = ['00','20', '40', '41', '60', '90'];
        $impostos    = ['99', '00', '98'];
        $modalBsCalc = ['1', '2', '3', '4'];
        
        if($CRT == '1') {
            if(!in_array($productTaxes['cstIcms'], $cstSN)) {
                throw new Exception\ParamsException("CSOSN {$productTaxes['cstIcms']} inválido ou não pertence ao regime simples nacional.");
            }
                
        } else {
            if(!in_array($productTaxes['cstIcms'], $cstRN)) {
                throw new Exception\ParamsException("CST-ICMS {$productTaxes['cstIcms']} inválido ou não pertence ao regime normal.");
            }
        }
        
        if(!in_array($productTaxes['cfopCode'], $cfop)) {
                throw new Exception\ParamsException("CFOP {$productTaxes['cfopCode']} inválido.");
        }
        
        if(!in_array($productTaxes['taxCode'], $impostos)) {
                throw new Exception\ParamsException("Imposto código: {$productTaxes['taxCode']} inválido. Códigos esperados: 99 - ICMS | 99 - ST | 98 - ISENTO. ");
        }
        
        if(!in_array($productTaxes['cstPisCofins'], $cstPisCof)) {
                throw new Exception\ParamsException("CST-PIS/COFINS {$productTaxes['cstPisCofins']} inválido.");
        }
        
        if(!in_array($productTaxes['icmsCalculationBasis'], $modalBsCalc)) {
                throw new Exception\ParamsException("Identificador de modalidade de base de calculo do ICMS inválido.");
        }
    }
    
    public function processCancelSale($params, $subsidiaryCode, $generalParams) {
        
        $cdcaixa      = isset($params['posCode']) ? $params['posCode'] : null;
        $saleDate     = $params['saleDate'];
        $nrAcessoNFCe = $params['chaveSefaz'];
        $idSubProjeto = $this->getIdSubProjeto();
        
        $canceledSale = $this->orderFactory->getSaleToCancel($subsidiaryCode, $cdcaixa, $saleDate, $nrAcessoNFCe, $idSubProjeto);
        
        if(empty($canceledSale)) {
            throw new Exception\ParamsException("Não foi encontrado venda com os parametros fornecidos.");
        }
        
        if(count($canceledSale) > 1) {
            throw new Exception\ParamsException("Encontrado mais de uma venda com os parametros fornececidos. Contate a Teknisa");
        }
        
        $canceledSale = $canceledSale[0];
        
        if($canceledSale['IDSITUVENDA'] == 'C' || $canceledSale['IDSTATUSNFCE'] == 'C') {
            throw new Exception\ParamsException("Venda já cancelada.");
        }
        
        $this->orderFactory->cancelSale($canceledSale['CDFILIAL'], $canceledSale['CDCAIXA'], $canceledSale['NRSEQVENDA']);
        $this->orderFactory->cancelSaleMoviment($canceledSale['CDFILIAL'], $canceledSale['CDCAIXA'], $canceledSale['NRSEQVENDA']);
        
        if(!empty($canceledSale['CDCLIENTE']) && !empty($canceledSale['CDCONSUMIDOR'])) {
            $movClie = $this->orderFactory->getMoviClieBySaleSeq($canceledSale['CDFILIAL'], $canceledSale['CDCAIXA'], $canceledSale['NRSEQVENDA'], $canceledSale['CDCLIENTE'], $canceledSale['CDCONSUMIDOR']);
            
            if(!empty($movClie) && $movClie['IDTIPMOCVLI'] != 'X') {
                $this->orderFactory->cancelMoviClie($canceledSale['CDFILIAL'], $canceledSale['CDCAIXA'], $canceledSale['NRSEQVENDA'], $canceledSale['CDCLIENTE'], $canceledSale['CDCONSUMIDOR'], $movClie['NRSEQMOVCLI']);
            }
        }
    }
    
    public function getsaleProducts($params, $page, $perPage) {
        $generalParams     = $params;
        $branch            = $this->checkParams($generalParams);
        $subsidiaryCode    = $branch['CDFILIAL'];
        $subsidiaryExtCode = isset($generalParams['subsidiaryExtCode']) ? $generalParams['subsidiaryExtCode'] : null;
        
        $dtInicio = date('d/m/Y', strtotime($branch['DTINICIAL']));
        $dtFinal  = date('d/m/Y', strtotime($branch['DTFINAL']));
        
        $sales = $this->orderFactory->getSalesByDay($subsidiaryCode, $dtInicio, $dtFinal);
        $return = [];
        
        if(empty($sales)) {
            throw new Exception\ParamsException("Não foi encontrado vendas para o período solicitado.");
        }
        
        foreach($sales as $sale) {
            $salesGroup = [
                        'id'                => $this->util->generateToken($subsidiaryCode, $sale['CDCAIXA'], $sale['NRSEQVENDA'], $sale['NRNOTAFISCALCE']),
                        'subsidiaryCode'    => $subsidiaryCode,
                        'subsidiaryExtCode' => $subsidiaryExtCode,
                        'subsidiaryName'    => $branch['NMFILIAL'],
                        'saleDate'          => $sale['DTVENDA'],
                        'saleCode'          => $sale['NRSEQVENDA'],
                        'cupomNumber'       => $sale['NRNOTAFISCALCE'],
                        'products'          => []
                    ];
                    
            $products = $this->productService->getsaleProducts($subsidiaryCode, $sale['CDCAIXA'], $sale['NRSEQVENDA']);
            
            if(empty($products)) {
                throw new Exception\ParamsException("Falha ao consultar produtos da venda: {$sale['NRSEQVENDA']} | caixa: {$sale['CDCAIXA']}.");
            }
            
            $salesGroup['products'] = $products;
            
            array_push($return, $salesGroup);
        }
        
        return $return;
    }
}